@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,6 +48,7 @@
 #include "MagickCore/memory_.h"
 #include "MagickCore/memory-private.h"
 #include "MagickCore/nt-base-private.h"
+#include "MagickCore/registry.h"
 #include "MagickCore/string-private.h"
 #include "MagickCore/timer.h"
 #include "MagickCore/timer-private.h"
@@ -68,6 +69,12 @@ static double
 static void
   StopTimer(TimerInfo *);
 
+/*
+  Static declarations.
+*/
+static ssize_t
+  date_precision = -1;
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -262,11 +269,26 @@ MagickExport ssize_t FormatMagickTime(const time_t time,const size_t length,
     utc_time;
 
   assert(timestamp != (char *) NULL);
+  if (date_precision == -1)
+    {
+      char
+        *limit;
+
+      date_precision=0;
+      limit=GetEnvironmentValue("MAGICK_DATE_PRECISION");
+      if (limit != (char *) NULL)
+        {
+          date_precision=StringToInteger(limit);
+          limit=DestroyString(limit);
+        }
+    }
   GetMagickUTCtime(&time,&utc_time);
   count=FormatLocaleString(timestamp,length,
     "%04d-%02d-%02dT%02d:%02d:%02d%+03d:00",utc_time.tm_year+1900,
     utc_time.tm_mon+1,utc_time.tm_mday,utc_time.tm_hour,utc_time.tm_min,
     utc_time.tm_sec,0);
+  if ((date_precision > 0) && (date_precision < (ssize_t) strlen(timestamp)))
+    timestamp[date_precision]='\0';
   return(count);
 }
 
@@ -325,26 +347,31 @@ MagickExport double GetElapsedTime(TimerInfo *time_info)
 */
 MagickExport time_t GetMagickTime(void)
 {
-  static const char
-    *source_date_epoch = (const char *) NULL;
+  static time_t
+    constant_magick_time = 0;
 
   static MagickBooleanType
-    epoch_initalized = MagickFalse;
+    epoch_initialized = MagickFalse;
 
-  if (epoch_initalized == MagickFalse)
+  if (epoch_initialized == MagickFalse)
     {
+      const char
+        *source_date_epoch;
+
       source_date_epoch=getenv("SOURCE_DATE_EPOCH");
-      epoch_initalized=MagickTrue;
-    }
-  if (source_date_epoch != (const char *) NULL)
-    {
-      time_t
-        epoch;
+      if (source_date_epoch != (const char *) NULL)
+        {
+          time_t
+            epoch;
 
-      epoch=(time_t) StringToDouble(source_date_epoch,(char **) NULL);
-      if ((epoch > 0) && (epoch <= time((time_t *) NULL)))
-        return(epoch);
+          epoch=(time_t) StringToDouble(source_date_epoch,(char **) NULL);
+          if ((epoch > 0) && (epoch <= time((time_t *) NULL)))
+            constant_magick_time=epoch;
+        }
+      epoch_initialized=MagickTrue;
     }
+  if (constant_magick_time != 0)
+    return(constant_magick_time);
   return(time((time_t *) NULL));
 }
 
@@ -453,6 +480,33 @@ MagickExport void ResetTimer(TimerInfo *time_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S e t M a g i c k D a t e P r e c i s i o n                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetMagickDatePrecision() sets the pseudo-random number generator secret key.
+%
+%  The format of the SetMagickDatePrecision method is:
+%
+%      void SetMagickDatePrecision(const unsigned long precision)
+%
+%  A description of each parameter follows:
+%
+%    o key: the date precision.
+%
+*/
+MagickPrivate void SetMagickDatePrecision(const unsigned long precision)
+{
+  date_precision=(ssize_t) precision;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   S t a r t T i m e r                                                       %
 %                                                                             %
 %                                                                             %
@@ -561,7 +615,7 @@ static double UserTime(void)
   return((double) (timer.tms_utime+timer.tms_stime)/sysconf(_SC_CLK_TCK));
 #else
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
-  return(NTUserTime());
+  return(NTElapsedTime());
 #else
   return((double) clock()/CLOCKS_PER_SEC);
 #endif
