@@ -1332,7 +1332,7 @@ static char *ParseEntities(char *xml,char **entities,int state)
   for (xml=p; ; )
   {
     while ((*xml != '\0') && (*xml != '&') && ((*xml != '%') ||
-           (state != '%')) && (isspace((int) ((unsigned char) *xml) == 0)))
+           (state != '%')) && (isspace((int) ((unsigned char) *xml)) == 0))
       xml++;
     if (*xml == '\0')
       break;
@@ -1442,7 +1442,7 @@ static char *ParseEntities(char *xml,char **entities,int state)
         }
       else
         if (((state == ' ') || (state == '*')) &&
-            (isspace((int) ((unsigned char) *xml) != 0)))
+            (isspace((int) ((unsigned char) *xml)) != 0))
           *(xml++)=' ';
         else
           xml++;
