@@ -244,7 +244,7 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
 
   Image
     *difference_image,
-    *image,
+    *image = (Image *) NULL,
     *reconstruct_image,
     *similarity_image;
 
