@@ -280,7 +280,7 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *image;
 
   JxlBasicInfo
-    basic_info = { 0 };
+    basic_info;
 
   JxlDecoder
     *jxl_info;
@@ -293,7 +293,7 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     memory_manager;
 
   JxlPixelFormat
-    pixel_format = { 0 };
+    pixel_format;
 
   MagickBooleanType
     status;
@@ -337,6 +337,8 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize JXL delegate library.
   */
+  memset(&basic_info,0,sizeof(basic_info));
+  memset(&pixel_format,0,sizeof(pixel_format));
   JXLSetMemoryManager(&memory_manager,&memory_manager_info,image,exception);
   jxl_info=JxlDecoderCreate(&memory_manager);
   if (jxl_info == (JxlDecoder *) NULL)
@@ -365,7 +367,7 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
           ThrowReaderException(CoderError,"MemoryAllocationFailed");
         }
     }
-  if (JxlDecoderSubscribeEvents(jxl_info,events_wanted) != JXL_DEC_SUCCESS)
+  if (JxlDecoderSubscribeEvents(jxl_info,(int) events_wanted) != JXL_DEC_SUCCESS)
     {
       if (runner != NULL)
         JxlThreadParallelRunnerDestroy(runner);
@@ -436,7 +438,7 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       case JXL_DEC_COLOR_ENCODING:
       {
         JxlColorEncoding
-          color_encoding = { 0 };
+          color_encoding;
 
         size_t
           profile_size;
@@ -444,9 +446,15 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         StringInfo
           *profile;
 
+        memset(&color_encoding,0,sizeof(color_encoding));
         JXLSetFormat(image,&pixel_format,exception);
+#if JPEGXL_NUMERIC_VERSION >= JPEGXL_COMPUTE_NUMERIC_VERSION(0,9,0)
+        jxl_status=JxlDecoderGetColorAsEncodedProfile(jxl_info,
+          JXL_COLOR_PROFILE_TARGET_DATA,&color_encoding);
+#else
         jxl_status=JxlDecoderGetColorAsEncodedProfile(jxl_info,&pixel_format,
           JXL_COLOR_PROFILE_TARGET_DATA,&color_encoding);
+#endif
         if (jxl_status == JXL_DEC_SUCCESS)
           {
             if (color_encoding.transfer_function == JXL_TRANSFER_FUNCTION_LINEAR)
@@ -519,14 +527,25 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         else
           if (jxl_status != JXL_DEC_ERROR)
             break;
+#if JPEGXL_NUMERIC_VERSION >= JPEGXL_COMPUTE_NUMERIC_VERSION(0,9,0)
+        jxl_status=JxlDecoderGetICCProfileSize(jxl_info,
+          JXL_COLOR_PROFILE_TARGET_ORIGINAL,&profile_size);
+#else
         jxl_status=JxlDecoderGetICCProfileSize(jxl_info,&pixel_format,
           JXL_COLOR_PROFILE_TARGET_ORIGINAL,&profile_size);
+#endif
         if (jxl_status != JXL_DEC_SUCCESS)
           break;
         profile=AcquireStringInfo(profile_size);
+#if JPEGXL_NUMERIC_VERSION >= JPEGXL_COMPUTE_NUMERIC_VERSION(0,9,0)
+        jxl_status=JxlDecoderGetColorAsICCProfile(jxl_info,
+          JXL_COLOR_PROFILE_TARGET_ORIGINAL,GetStringInfoDatum(profile),
+          profile_size);
+#else
         jxl_status=JxlDecoderGetColorAsICCProfile(jxl_info,&pixel_format,
           JXL_COLOR_PROFILE_TARGET_ORIGINAL,GetStringInfoDatum(profile),
           profile_size);
+#endif
         if (jxl_status == JXL_DEC_SUCCESS)
           (void) SetImageProfile(image,"icm",profile,exception);
         profile=DestroyStringInfo(profile);
@@ -854,7 +873,7 @@ static inline float JXLGetDistance(const ImageInfo *image_info)
     return(1.0f);
   if (image_info->quality >= 30)
     return(0.1f+(float) (100-MagickMin(100,image_info->quality))*0.09f);
-  return(6.24f+(float) pow(2.5f,(30.0-image_info->quality)/5.0f)/6.25f);
+  return(6.24f+(float) pow(2.5f,(30.0-image_info->quality)/5.0)/6.25f);
 }
 
 static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
@@ -868,7 +887,7 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
     *xmp_profile = (StringInfo *) NULL;
 
   JxlBasicInfo
-    basic_info = { 0 };
+    basic_info;
 
   JxlEncoder
     *jxl_info;
@@ -880,13 +899,13 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
     jxl_status;
 
   JxlFrameHeader
-    frame_header = { 0 };
+    frame_header;
 
   JxlMemoryManager
     memory_manager;
 
   JxlPixelFormat
-    pixel_format = { 0 };
+    pixel_format;
 
   MagickBooleanType
     status;
@@ -926,6 +945,9 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
   /*
     Initialize JXL delegate library.
   */
+  memset(&basic_info,0,sizeof(basic_info));
+  memset(&frame_header,0,sizeof(frame_header));
+  memset(&pixel_format,0,sizeof(pixel_format));
   JXLSetMemoryManager(&memory_manager,&memory_manager_info,image,exception);
   jxl_info=JxlEncoderCreate(&memory_manager);
   if (jxl_info == (JxlEncoder *) NULL)
@@ -1063,7 +1085,7 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
      sizeof(char));
   if (IsGrayColorspace(image->colorspace) != MagickFalse)
     bytes_per_row=image->columns*
-      (((image->alpha_trait &= BlendPixelTrait) != 0) ? 2 : 1)*
+      (((image->alpha_trait & BlendPixelTrait) != 0) ? 2 : 1)*
       ((pixel_format.data_type == JXL_TYPE_FLOAT) ? sizeof(float) :
        (pixel_format.data_type == JXL_TYPE_UINT16) ? sizeof(short) :
        sizeof(char));
