@@ -17,7 +17,7 @@
 %                                 March 2011                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2011 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -201,7 +201,7 @@ static MagickBooleanType IsWEBPImageLossless(const unsigned char *stream,
     chunk_size=ReadWebPLSBWord(stream+offset+TAG_SIZE);
     if (chunk_size > MAX_CHUNK_PAYLOAD)
       break;
-    chunk_size_pad=(CHUNK_HEADER_SIZE+chunk_size+1) & ~1;
+    chunk_size_pad=(CHUNK_HEADER_SIZE+chunk_size+1) & (unsigned int) ~1;
     if (memcmp(stream+offset,VP8_CHUNK_HEADER,VP8_CHUNK_HEADER_SIZE) == 0)
       return(*(stream+offset+VP8_CHUNK_HEADER_SIZE) == LOSSLESS_FLAG ?
         MagickTrue : MagickFalse);
@@ -219,7 +219,7 @@ static int FillBasicWEBPInfo(Image *image,const uint8_t *stream,size_t length,
   int
     webp_status;
 
-  webp_status=WebPGetFeatures(stream,length,features);
+  webp_status=(int) WebPGetFeatures(stream,length,features);
 
   if (webp_status != VP8_STATUS_OK)
     return(webp_status);
@@ -233,15 +233,15 @@ static int FillBasicWEBPInfo(Image *image,const uint8_t *stream,size_t length,
   return(webp_status);
 }
 
-static int ReadSingleWEBPImage(Image *image,const uint8_t *stream,
-  size_t length,WebPDecoderConfig *configure,ExceptionInfo *exception,
-  MagickBooleanType is_first)
+static int ReadSingleWEBPImage(const ImageInfo *image_info,Image *image,
+  const uint8_t *stream,size_t length,WebPDecoderConfig *configure,
+  ExceptionInfo *exception,MagickBooleanType is_first)
 {
   int
     webp_status;
 
-  unsigned char
-    *p;
+  MagickBooleanType
+    status;
 
   size_t
     canvas_width,
@@ -254,12 +254,12 @@ static int ReadSingleWEBPImage(Image *image,const uint8_t *stream,
     y_offset,
     y;
 
+  unsigned char
+    *p;
+
   WebPDecBuffer
     *magick_restrict webp_image;
 
-  MagickBooleanType
-    status;
-
   webp_image=&configure->output;
   if (is_first != MagickFalse)
     {
@@ -292,7 +292,10 @@ static int ReadSingleWEBPImage(Image *image,const uint8_t *stream,
   if (IsWEBPImageLossless((unsigned char *) stream,length) != MagickFalse)
     image->quality=100;
 
-  webp_status=WebPDecode(stream,length,configure);
+  if (image_info->ping != MagickFalse)
+    return(webp_status);
+
+  webp_status=(int) WebPDecode(stream,length,configure);
   if (webp_status != VP8_STATUS_OK)
     return(webp_status);
 
@@ -310,8 +313,8 @@ static int ReadSingleWEBPImage(Image *image,const uint8_t *stream,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if ((x >= x_offset && x < (ssize_t) (x_offset+image_width)) &&
-          (y >= y_offset && y < (ssize_t) (y_offset+image_height)))
+      if (((x >= x_offset) && (x < (x_offset+(ssize_t) image_width))) &&
+          ((y >= y_offset) && (y < (y_offset+(ssize_t) image_height))))
         {
           SetPixelRed(image,ScaleCharToQuantum(*p++),q);
           SetPixelGreen(image,ScaleCharToQuantum(*p++),q);
@@ -445,7 +448,7 @@ static int ReadAnimatedWEBPImage(const ImageInfo *image_info,Image *image,
     mux=WebPMuxCreate(&data,0);
     status=WebPMuxGetAnimationParams(mux,&params);
     if (status >= 0)
-      image->iterations=(ssize_t) params.loop_count;
+      image->iterations=(size_t) params.loop_count;
     WebPMuxDelete(mux);
   }
   demux=WebPDemux(&data);
@@ -462,15 +465,17 @@ static int ReadAnimatedWEBPImage(const ImageInfo *image_info,Image *image,
             CloneImageProperties(image,original_image);
             image->page.x=(ssize_t) iter.x_offset;
             image->page.y=(ssize_t) iter.y_offset;
-            webp_status=ReadSingleWEBPImage(image,iter.fragment.bytes,
-              iter.fragment.size,configure,exception,MagickFalse);
+            webp_status=ReadSingleWEBPImage(image_info,image,
+              iter.fragment.bytes,iter.fragment.size,configure,exception,
+              MagickFalse);
           }
         else
           {
             image->page.x=(ssize_t) iter.x_offset;
             image->page.y=(ssize_t) iter.y_offset;
-            webp_status=ReadSingleWEBPImage(image,iter.fragment.bytes,
-              iter.fragment.size,configure,exception,MagickTrue);
+            webp_status=ReadSingleWEBPImage(image_info,image,
+              iter.fragment.bytes,iter.fragment.size,configure,exception,
+              MagickTrue);
           }
         if (webp_status != VP8_STATUS_OK)
           break;
@@ -556,7 +561,7 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
   count=ReadBlob(image,12,header);
   if (count != 12)
     ThrowWEBPException(CorruptImageError,"InsufficientImageDataInFile");
-  status=IsWEBP(header,(ssize_t) count);
+  status=IsWEBP(header,(size_t) count);
   if (status == MagickFalse)
     ThrowWEBPException(CorruptImageError,"CorruptImage");
   length=(size_t) (ReadWebPLSBWord(header+4)+8);
@@ -582,7 +587,8 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
       webp_status=VP8_STATUS_UNSUPPORTED_FEATURE;
 #endif
     } else {
-      webp_status=ReadSingleWEBPImage(image,stream,length,&configure,exception,MagickFalse);
+      webp_status=ReadSingleWEBPImage(image_info,image,stream,length,
+        &configure,exception,MagickFalse);
     }
   }
 
@@ -790,8 +796,8 @@ static const char * WebPErrorCodeMessage(WebPEncodingError error_code)
 }
 
 static MagickBooleanType WriteSingleWEBPPicture(const ImageInfo *image_info,
-  Image *image,const WebPConfig *configure,WebPPicture *picture,
-  MemoryInfo **memory_info,ExceptionInfo *exception)
+  Image *image,WebPPicture *picture,MemoryInfo **memory_info,
+  ExceptionInfo *exception)
 {
   MagickBooleanType
     status;
@@ -852,14 +858,6 @@ static MagickBooleanType WriteSingleWEBPPicture(const ImageInfo *image_info,
     if (status == MagickFalse)
       break;
   }
-
-  if (status != MagickFalse)
-    status=(MagickBooleanType) WebPEncode(configure,picture);
-
-  if (status == MagickFalse)
-    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
-      WebPErrorCodeMessage(picture->error_code),"`%s'",image->filename);
-
   return(status);
 }
 
@@ -882,8 +880,14 @@ static MagickBooleanType WriteSingleWEBPImage(const ImageInfo *image_info,
   picture.writer=WebPMemoryWrite;
   picture.custom_ptr=writer;
 
-  status=WriteSingleWEBPPicture(image_info,image,configure,&picture,
-    &memory_info,exception);
+  status=WriteSingleWEBPPicture(image_info,image,&picture,&memory_info,
+    exception);
+  if (status != MagickFalse)
+    status=(MagickBooleanType) WebPEncode(configure,&picture);
+  if (status == MagickFalse)
+    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
+      WebPErrorCodeMessage(picture.error_code),"`%s'",image->filename);
+
   if (memory_info != (MemoryInfo *) NULL)
     memory_info=RelinquishVirtualMemory(memory_info);
   WebPPictureFree(&picture);
@@ -952,11 +956,14 @@ static MagickBooleanType WriteAnimatedWEBPImage(const ImageInfo *image_info,
         break;
       }
 
-    webp_status=(int) WriteSingleWEBPPicture(image_info,frame,configure,
-      &picture,&memory_info,exception);
+    webp_status=(int) WriteSingleWEBPPicture(image_info,frame,&picture,
+      &memory_info,exception);
     if (webp_status != 0)
       webp_status=WebPAnimEncoderAdd(enc,&picture,(int) frame_timestamp,
         configure);
+    else
+      (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
+        WebPErrorCodeMessage(picture.error_code),"`%s'",image->filename);
     if (memory_info != (MemoryInfo *) NULL)
       (void) AppendValueToLinkedList(memory_info_list,memory_info);
     WebPPictureFree(&picture);
