@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -207,11 +207,11 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -430,7 +430,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate the pixel buffer.
   */
-  length=(size_t) ximage->bytes_per_line*ximage->height;
+  length=(size_t) (ximage->bytes_per_line*ximage->height);
   if (CheckOverflowException(length,ximage->bytes_per_line,ximage->height))
     {
       if (header.ncolors != 0)
@@ -444,7 +444,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         extent;
 
       extent=length;
-      length*=ximage->depth;
+      length*=(size_t) ximage->depth;
       if (CheckOverflowException(length,extent,ximage->depth))
         {
           if (header.ncolors != 0)
@@ -544,16 +544,16 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (x=0; x < (ssize_t) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
-              index=(Quantum) ConstrainColormapIndex(image,(ssize_t) (pixel >>
-                red_shift) & red_mask,exception);
+              index=(Quantum) ConstrainColormapIndex(image,(ssize_t) ((pixel >>
+                red_shift) & red_mask),exception);
               SetPixelRed(image,ScaleShortToQuantum(
                 colors[(ssize_t) index].red),q);
-              index=(Quantum) ConstrainColormapIndex(image,(ssize_t) (pixel >>
-                green_shift) & green_mask,exception);
+              index=(Quantum) ConstrainColormapIndex(image,(ssize_t) ((pixel >>
+                green_shift) & green_mask),exception);
               SetPixelGreen(image,ScaleShortToQuantum(
                 colors[(ssize_t) index].green),q);
-              index=(Quantum) ConstrainColormapIndex(image,(ssize_t) (pixel >>
-                blue_shift) & blue_mask,exception);
+              index=(Quantum) ConstrainColormapIndex(image,(ssize_t) ((pixel >>
+                blue_shift) & blue_mask),exception);
               SetPixelBlue(image,ScaleShortToQuantum(
                 colors[(ssize_t) index].blue),q);
               q+=GetPixelChannels(image);
@@ -794,10 +794,10 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
